@@ -1,33 +1,23 @@
 from __future__ import annotations
 
-import json
-import os
 from dataclasses import dataclass
 from functools import lru_cache
-from types import MappingProxyType
-from typing import Any, Callable, Type
+from typing import Any
 
-from pythonbible.bible.bible_parser import BibleParser
-from pythonbible.bible.osis.parser import OSISParser
+from pythonbible.bible.bibles import get_bible
+from pythonbible.bible.titles import LONG_TITLES
+from pythonbible.bible.titles import SHORT_TITLES
 from pythonbible.books import Book
-from pythonbible.converter import (
-    convert_references_to_verse_ids,
-    convert_verse_ids_to_references,
-)
-from pythonbible.errors import (
-    InvalidVerseError,
-    MissingBookFileError,
-    MissingVerseFileError,
-)
+from pythonbible.converter import convert_references_to_verse_ids
+from pythonbible.converter import convert_verse_ids_to_references
+from pythonbible.errors import MissingBookFileError
 from pythonbible.normalized_reference import NormalizedReference
-from pythonbible.verses import (
-    VERSE_IDS,
-    get_book_chapter_verse,
-    get_number_of_chapters,
-    get_number_of_verses,
-    is_single_chapter_book,
-)
-from pythonbible.versions import DEFAULT_VERSION, Version
+from pythonbible.verses import get_book_chapter_verse
+from pythonbible.verses import get_number_of_chapters
+from pythonbible.verses import get_number_of_verses
+from pythonbible.verses import is_single_chapter_book
+from pythonbible.versions import DEFAULT_VERSION
+from pythonbible.versions import Version
 
 
 @dataclass
@@ -36,31 +26,6 @@ class BookTitles:
     short_title: str
 
 
-VERSION_MAP: dict[Version, Type[BibleParser]] = MappingProxyType(
-    {
-        Version.AMERICAN_STANDARD: OSISParser,
-        Version.KING_JAMES: OSISParser,
-    },
-)
-
-
-def get_parser(**kwargs: Any) -> BibleParser:
-    version: Version = kwargs.get("version", DEFAULT_VERSION)
-    version_map: dict[Version, Type[BibleParser]] = kwargs.get(
-        "version_map",
-        VERSION_MAP,
-    )
-    return version_map.get(version, OSISParser)(version)
-
-
-DEFAULT_PARSER: BibleParser = get_parser()
-
-CURRENT_FOLDER: str = os.path.dirname(os.path.realpath(__file__))
-DATA_FOLDER: str = os.path.join(CURRENT_FOLDER, "bible", "data")
-verse_texts: dict[Version, dict[int, str]] = {}
-book_titles: dict[Version, dict[Book, BookTitles]] = {}
-
-
 # TODO - handle Psalms vs Psalm appropriately
 # TODO - handle single chapter books appropriately (e.g. Obadiah 1-4 rather than
 #        Obadiah 1:1-4)
@@ -221,10 +186,7 @@ def _get_book_title(book: Book, include_books: bool = True, **kwargs: Any) -> st
 
     version: Version = kwargs.get("version", DEFAULT_VERSION)
     full_title: bool = kwargs.get("full_title", False)
-    version_book_titles: BookTitles | None = get_book_titles(book, version)
-
-    if not version_book_titles:
-        return ""
+    version_book_titles: BookTitles = get_book_titles(book, version)
 
     return (
         version_book_titles.long_title
@@ -361,7 +323,6 @@ def _does_reference_include_all_verses_in_end_book(
     return reference.end_verse == get_number_of_verses(reference.end_book, max_chapters)
 
 
-# TODO - rewrite this to not need a parser
 def format_scripture_text(verse_ids: list[int], **kwargs: Any) -> str:
     """
     Return the formatted scripture text for the given list of verse IDs.
@@ -375,124 +336,83 @@ def format_scripture_text(verse_ids: list[int], **kwargs: Any) -> str:
     full_title: bool = kwargs.get("full_title", False)
     format_type: str = kwargs.get("format_type", "html")
     include_verse_numbers: bool = kwargs.get("include_verse_numbers", True)
-    parser: BibleParser = kwargs.get("parser", DEFAULT_PARSER)
+    version: Version = kwargs.get("version", DEFAULT_VERSION)
 
-    if one_verse_per_paragraph or len(verse_ids) == 1:
-        return _format_scripture_text_verse_by_verse(
-            verse_ids,
-            parser.version,
-            full_title,
-            format_type,
-            include_verse_numbers,
-        )
+    is_html: bool = format_type == "html"
 
-    return _format_scripture_text_with_parser(
-        verse_ids,
-        parser,
-        full_title,
-        format_type,
-        include_verse_numbers,
-    )
+    bible_type = "html" if is_html else "plain_text"
 
+    if not include_verse_numbers:
+        bible_type += "_readers"
+
+    bible = get_bible(version, bible_type)
 
-def _format_scripture_text_verse_by_verse(
-    verse_ids: list[int],
-    version: Version,
-    full_title: bool,
-    format_type: str,
-    include_verse_numbers: bool,
-) -> str:
     verse_ids.sort()
     text: str = ""
     current_book: Book | None = None
     current_chapter: int | None = None
+    previous_verse_number: int | None = None
+    current_start_verse: int | None = None
+    current_end_verse: int | None = None
 
     for verse_id in verse_ids:
         book, chapter_number, verse_number = get_book_chapter_verse(verse_id)
 
+        if (
+            one_verse_per_paragraph
+            or book != current_book
+            or chapter_number != current_chapter
+            or previous_verse_number is None
+            or verse_number != previous_verse_number + 1
+        ):
+            if current_start_verse and current_end_verse:
+                verse_text = bible.get_scripture(current_start_verse, current_end_verse)
+                text += _format_paragraph(verse_text)
+
+            current_start_verse = verse_id
+
+        current_end_verse = verse_id
+        previous_verse_number = verse_number
+
         if book != current_book:
             current_book = book
             current_chapter = chapter_number
-            version_book_titles: BookTitles | None = get_book_titles(book, version)
-
-            if version_book_titles:
-                title: str = (
-                    version_book_titles.long_title
-                    if full_title
-                    else version_book_titles.short_title
-                )
-                text += _format_title(title, format_type, not text)
-
-            text += _format_chapter(chapter_number, format_type)
-
+            version_book_titles: BookTitles = get_book_titles(book, version)
+            title: str = (
+                version_book_titles.long_title
+                if full_title
+                else version_book_titles.short_title
+            )
+            text += _format_title(title, is_html, not text)
+            text += _format_chapter(chapter_number, is_html)
         elif chapter_number != current_chapter:
             current_chapter = chapter_number
-            text += _format_chapter(chapter_number, format_type)
+            text += _format_chapter(chapter_number, is_html)
 
-        verse_text: str | None = get_verse_text(verse_id, version)
-
-        if include_verse_numbers:
-            verse_text = f"{verse_number}. {verse_text}"
-
-        text += _format_paragraph(verse_text, format_type)
+    if current_start_verse and current_end_verse:
+        verse_text = bible.get_scripture(current_start_verse, current_end_verse)
+        text += _format_paragraph(verse_text)
 
     return text
 
 
-def _format_scripture_text_with_parser(
-    verse_ids: list[int],
-    parser: BibleParser,
-    full_title: bool,
-    format_type: str,
-    include_verse_numbers: bool,
-) -> str:
-    title_function: Callable[[Any], Any] = (
-        parser.get_book_title if full_title else parser.get_short_book_title
-    )
-    text: str = ""
-
-    paragraphs: Any = parser.get_scripture_passage_text(
-        verse_ids,
-        include_verse_number=include_verse_numbers,
-    )
-
-    for book, chapters in paragraphs.items():
-        title: str = title_function(book)
-        text += _format_title(title, format_type, not text)
-
-        for chapter, chapter_paragraphs in chapters.items():
-            text += _format_chapter(chapter, format_type)
-            text += "".join(
-                _format_paragraph(paragraph, format_type)
-                for paragraph in chapter_paragraphs
-            )
-
-    return text
-
-
-def _format_title(title: str, format_type: str, is_first_book: bool) -> str:
-    if format_type == "html":
+def _format_title(title: str, is_html: bool, is_first_book: bool) -> str:
+    if is_html:
         return f"<h1>{title}</h1>\n"
 
     return f"{title}\n\n" if is_first_book else f"\n\n{title}\n\n"
 
 
-def _format_chapter(chapter: int, format_type: str) -> str:
-    if format_type == "html":
-        return f"<h2>Chapter {chapter}</h2>\n"
-
-    return f"Chapter {chapter}\n\n"
+def _format_chapter(chapter: int, is_html: bool) -> str:
+    return f"<h2>Chapter {chapter}</h2>\n" if is_html else f"Chapter {chapter}\n\n"
 
 
-def _format_paragraph(paragraph: str | None, format_type: str) -> str:
-    if format_type == "html":
-        return f"<p>{paragraph}</p>\n"
-
-    return f"   {paragraph}\n"
+def _format_paragraph(paragraph: str) -> str:
+    return f"{paragraph}\n"
 
 
 @lru_cache()
-def get_verse_text(verse_id: int, version: Version = DEFAULT_VERSION) -> str | None:
+def get_verse_text(verse_id: int, version: Version = DEFAULT_VERSION) -> str:
     """
     Return the scripture text of the given verse id and version of the Bible.
 
@@ -507,43 +427,12 @@ def get_verse_text(verse_id: int, version: Version = DEFAULT_VERSION) -> str | N
     :raises MissingVerseFileError: if the verse file for the given verse_id and version
                                    does not exist
     """
-    if verse_id not in VERSE_IDS:
-        raise InvalidVerseError(verse_id=verse_id)
-
-    try:
-        version_verse_texts: dict[int, str] = _get_version_verse_texts(version)
-    except FileNotFoundError as file_not_found_error:
-        raise MissingVerseFileError(file_not_found_error)
-
-    return version_verse_texts.get(verse_id)
+    bible = get_bible(version, "plain_text_readers")
+    return bible.get_scripture(verse_id, verse_id)
 
 
 @lru_cache()
-def _get_version_verse_texts(version: Version) -> dict[int, str]:
-    version_verse_texts: dict[int, str] | None = verse_texts.get(version)
-
-    if version_verse_texts is None:
-        json_filename: str = os.path.join(
-            DATA_FOLDER,
-            version.value.lower(),
-            "verses.json",
-        )
-        version_verse_texts = {}
-
-        with open(json_filename, "r", encoding="utf-8") as json_file:
-            for verse_id, verse_text in json.load(json_file).items():
-                version_verse_texts[int(verse_id)] = verse_text
-
-        verse_texts[version] = version_verse_texts
-
-    return version_verse_texts
-
-
-@lru_cache()
-def get_book_titles(
-    book: Book,
-    version: Version = DEFAULT_VERSION,
-) -> BookTitles | None:
+def get_book_titles(book: Book, version: Version = DEFAULT_VERSION) -> BookTitles:
     """
     Return the book titles for the given Book and optional Version.
 
@@ -552,35 +441,22 @@ def get_book_titles(
     :param version: a version of the Bible, defaults to American Standard
     :type version: Version
     :return: the long and short titles of the given book and version
-    :rtype: BookTitles | None
+    :rtype: BookTitles
     :raises MissingBookFileError: if the book file for the given book and version does
                                   not exist
     """
-    try:
-        version_book_titles: dict[Book, BookTitles] = _get_version_book_titles(version)
-    except FileNotFoundError as file_not_found_error:
-        raise MissingBookFileError(file_not_found_error)
+    short_titles, long_titles = _get_version_book_titles(version)
+    short_title = short_titles.get(book, book.title)
+    long_title = long_titles.get(book, book.title)
 
-    return version_book_titles.get(book)
+    return BookTitles(long_title, short_title)
 
 
 @lru_cache()
-def _get_version_book_titles(version: Version) -> dict[Book, BookTitles]:
-    version_book_titles: dict[Book, BookTitles] | None = book_titles.get(version)
-
-    if version_book_titles is None:
-        json_filename: str = os.path.join(
-            DATA_FOLDER,
-            version.value.lower(),
-            "books.json",
-        )
-        version_book_titles = {}
-
-        with open(json_filename, "r", encoding="utf-8") as json_file:
-            for book_id, titles in json.load(json_file).items():
-                book: Book = Book(int(book_id))
-                version_book_titles[book] = BookTitles(titles[0], titles[1])
-
-        book_titles[version] = version_book_titles
-
-    return version_book_titles
+def _get_version_book_titles(
+    version: Version,
+) -> tuple[dict[Book, str], dict[Book, str]]:
+    try:
+        return SHORT_TITLES[version], LONG_TITLES[version]
+    except KeyError as key_error:
+        raise MissingBookFileError(key_error) from key_error
